USE XMLDB_2014;
GO

-- na poczatek co mamy - tabela bez indeksu
WITH XMLNAMESPACES (
	'http://www.mediawiki.org/xml/export-0.10/' AS x
)
SELECT TOP 1
	title		= T.c.value('x:title[1]', 'VARCHAR(500)'),
	timestamp	= T.c.value('x:revision[1]/x:timestamp[1]', 'VARCHAR(20)'),
	comment		= T.c.value('x:revision[1]/x:comment[1]', 'VARCHAR(1000)'),
	text		= T.c.value('x:revision[1]/x:text[1]', 'VARCHAR(MAX)')
FROM dbo.WikiXML_NoIndex w

	CROSS APPLY w.XMLData.nodes('/x:mediawiki/x:page') T(c)
;

-- zapytaj o przedzialy czasu; dodajemy CTE, bo nie mozna 
-- grupowac po wyrazeniach XML
-- 50s, 446659 rekordow
-- estimated cost: 1218.25
WITH XMLNAMESPACES (
	'http://www.mediawiki.org/xml/export-0.10/' AS x
),
Dane AS (
SELECT
	timestamp	= T.c.value('x:revision[1]/x:timestamp[1]', 'DATETIME2(0)')
FROM dbo.WikiXML_NoIndex w

	CROSS APPLY w.XMLData.nodes('/x:mediawiki/x:page') T(c)
)
SELECT
	timestamp,
	ile			= COUNT(*)
FROM Dane
GROUP BY
	timestamp
;

-- utworz indeks
CREATE PRIMARY XML INDEX PXI_XMLData
	ON dbo.WikiXML(XMLData)
;
GO
/*
Table 'dbo.WikiXML' needs to have a clustered primary key 
with less than 16 columns in it in order 
to create a primary XML index on it.
*/

ALTER TABLE dbo.WikiXML ADD
	CONSTRAINT PK_WikiXML_Id
	PRIMARY KEY CLUSTERED (Id)
;
GO

-- powtarzamy indeks
-- 2m14s
CREATE PRIMARY XML INDEX PXI_XMLData
	ON dbo.WikiXML(XMLData)
;
GO

-- ponawiamy zapytanie (na tabeli z indeksem)
WITH XMLNAMESPACES (
	'http://www.mediawiki.org/xml/export-0.10/' AS x
),
Dane AS (
SELECT
	timestamp	= T.c.value('x:revision[1]/x:timestamp[1]', 'DATETIME2(0)')
FROM dbo.WikiXML w

	CROSS APPLY w.XMLData.nodes('/x:mediawiki/x:page') T(c)
)
SELECT
	timestamp,
	ile			= COUNT(*)
FROM Dane
GROUP BY
	timestamp
;
-- 26s - przyspiesza dwukrotnie
-- subtree cost: 29 464 600

-- jak taki indeks wyglada?
SELECT * FROM sys.internal_tables
SELECT * FROM sys.indexes WHERE object_id = 331148225 --type = 3

EXEC sp_help 'sys.xml_index_nodes_331148225_256000'

-- =======
:CONNECT ADMIN:WIN-R1EPJQ390D6\SQL2014
--SELECT COUNT(*)	-- 13 836 009
--FROM XMLDB_2014.sys.xml_index_nodes_331148225_256000

SELECT TOP 50 *
FROM XMLDB_2014.sys.xml_index_nodes_331148225_256000

-- =======

-- indeksy secondary
-- PATH: 1m43s
CREATE XML INDEX IX_XMLData_PATH
	ON dbo.WikiXML(XMLData)
	USING XML INDEX PXI_XMLData
	FOR PATH
;
GO

-- ponawiamy zapytanie (na tabeli z indeksem PATH)
WITH XMLNAMESPACES (
	'http://www.mediawiki.org/xml/export-0.10/' AS x
),
Dane AS (
SELECT
	timestamp	= T.c.value('x:revision[1]/x:timestamp[1]', 'DATETIME2(0)')
FROM dbo.WikiXML w

	CROSS APPLY w.XMLData.nodes('/x:mediawiki/x:page') T(c)
)
SELECT
	timestamp,
	ile			= COUNT(*)
FROM Dane
GROUP BY
	timestamp
;

-- przerwane po minucie
-- plan zapytania zawiera index seek (PATH)
-- nie pomaga tez przepisanie sciezki
-- timestamp	= T.c.value('(x:revision/x:timestamp)[1]', 'DATETIME2(0)')



-- PATH przyspiesza glownie zapytania exist()
-- (spoiler alert: ale nie w tym przypadku)
WITH XMLNAMESPACES (
	'http://www.mediawiki.org/xml/export-0.10/' AS x
),
Dane AS (
SELECT
	timestamp	= T.c.value('(x:revision/x:timestamp)[1]', 'DATETIME2(0)')
FROM dbo.WikiXML w

	CROSS APPLY w.XMLData.nodes('/x:mediawiki/x:page') T(c)

WHERE
	1 = 1
	AND T.c.exist('x:revision/x:contributor/x:username[.="Ignasiak"]') = 1
)
SELECT
	timestamp,
	ile			= COUNT(*)
FROM Dane
GROUP BY
	timestamp
;

-- estimated cost: 597 391
-- przerwane po 1m30s


-- ========== SELECTIVE INDEX

USE XMLDB_2014;
GO

ALTER TABLE dbo.WikiXML_NoIndex ADD
	CONSTRAINT PK_WikiXML_NoIndex_Id
	PRIMARY KEY CLUSTERED (Id)
;
GO

CREATE SELECTIVE XML INDEX
	SXI_Selective_Contributor
ON dbo.WikiXML_NoIndex(XMLData)
WITH XMLNAMESPACES (
	'http://www.mediawiki.org/xml/export-0.10/' AS x
)
FOR (
    pathContributor = '/x:mediawiki/x:page/x:revision/x:contributor/x:username'
)
GO

-- ================
:CONNECT ADMIN:WIN-R1EPJQ390D6\SQL2014
SELECT TOP 50 *
FROM XMLDB_2014.sys.xml_sxi_table_523148909_256000
-- ================

WITH XMLNAMESPACES (
	'http://www.mediawiki.org/xml/export-0.10/' AS x
),
Dane AS (
SELECT
	timestamp	= T.c.value('(x:revision/x:timestamp)[1]', 'DATETIME2(0)')
FROM dbo.WikiXML_NoIndex w

	CROSS APPLY w.XMLData.nodes('/x:mediawiki/x:page') T(c)

WHERE
	1 = 1
	AND T.c.exist('x:revision/x:contributor/x:username[.="Ignasiak"]') = 1
)
SELECT
	timestamp,
	ile			= COUNT(*)
FROM Dane
GROUP BY
	timestamp
;

-- 42s
-- estimated cost: 7.01126

-- full path
WITH XMLNAMESPACES (
	'http://www.mediawiki.org/xml/export-0.10/' AS x
),
Dane AS (
SELECT
	timestamp	= T.c.value('(x:revision/x:timestamp)[1]', 'DATETIME2(0)')
FROM dbo.WikiXML_NoIndex w

	CROSS APPLY w.XMLData.nodes('/x:mediawiki/x:page') T(c)

WHERE
	1 = 1
	AND T.c.exist('/x:mediawiki/x:page/x:revision/x:contributor/x:username[.="Ignasiak"]') = 1
)
SELECT
	timestamp,
	ile			= COUNT(*)
FROM Dane
GROUP BY
	timestamp
;

-- 41s
-- estimated cost 818.117

