-- Apply best practices to XQuery
-- Get title and all personae from Shakepeare plays
-- Run S:\Scripts\LoadShakespeare.ps1 before starting demos
-- Ctrl+M for Actual Query Plan

SET STATISTICS IO, TIME ON

USE XMLDB_2014;
GO

-- original query
SELECT
	Tytul	= T.c.value('../../TITLE[1]', 'VARCHAR(100)'),
	Postac	= T.c.value('.[1]', 'VARCHAR(100)')
FROM dbo.PowerShellStuff s

	CROSS APPLY s.XMLStuff.nodes('/PLAY/PERSONAE/PERSONA') T(c)

ORDER BY
	T.c.value('../../TITLE[1]', 'VARCHAR(100)')

-- estimated subtree cost: 68053.1
-- CPU time = 171 ms,  elapsed time = 274 ms.

-- use text()
SELECT
	Tytul	= T.c.value('../../TITLE[1]', 'VARCHAR(100)'),
	Postac	= T.c.value('text()[1]', 'VARCHAR(100)')
FROM dbo.PowerShellStuff s

	CROSS APPLY s.XMLStuff.nodes('/PLAY/PERSONAE/PERSONA') T(c)

ORDER BY
	T.c.value('../../TITLE[1]', 'VARCHAR(100)')

-- estimated subtree cost: 61010.5
-- CPU time = 155 ms,  elapsed time = 113 ms.

-- use text() everywhere
SELECT
	Tytul	= T.c.value('../../TITLE[1]/text()[1]', 'VARCHAR(100)'),
	Postac	= T.c.value('text()[1]', 'VARCHAR(100)')
FROM dbo.PowerShellStuff s

	CROSS APPLY s.XMLStuff.nodes('/PLAY/PERSONAE/PERSONA') T(c)

ORDER BY
	T.c.value('../../TITLE[1]/text()[1]', 'VARCHAR(100)')

-- estimated subtree cost: 60731.6
-- CPU time = 125 ms,  elapsed time = 183 ms.

-- use path in parenthesis
SELECT
	Tytul	= T.c.value('(../../TITLE/text())[1]', 'VARCHAR(100)'),
	Postac	= T.c.value('(text())[1]', 'VARCHAR(100)')
FROM dbo.PowerShellStuff s

	CROSS APPLY s.XMLStuff.nodes('/PLAY/PERSONAE/PERSONA') T(c)

ORDER BY
	T.c.value('(../../TITLE/text())[1]', 'VARCHAR(100)')

-- estimated subtree cost: 66663.5
-- CPU time = 13422 ms,  elapsed time = 6927 ms.

-- avoid parent axis (..)
SELECT
	Tytul	= P.c.value('TITLE[1]/text()[1]', 'VARCHAR(100)'),
	Postac	= T.c.value('text()[1]', 'VARCHAR(100)')
FROM dbo.PowerShellStuff s

	CROSS APPLY s.XMLStuff.nodes('/PLAY') P(c)

	CROSS APPLY P.c.nodes('PERSONAE/PERSONA') T(c)

ORDER BY
	P.c.value('TITLE[1]/text()[1]', 'VARCHAR(100)')

-- estimated subtree cost: 40012.4
-- CPU time = 250 ms,  elapsed time = 186 ms.


-- double evaluation
SELECT
	Francja	= CASE
				WHEN T.c.value('text()[1]', 'VARCHAR(100)') LIKE '%FRANCE%'
				THEN T.c.value('text()[1]', 'VARCHAR(100)')
				ELSE ''
			END,
	Tytul	= P.c.value('TITLE[1]/text()[1]', 'VARCHAR(100)'),
	Postac	= T.c.value('text()[1]', 'VARCHAR(100)')
FROM dbo.PowerShellStuff s

	CROSS APPLY s.XMLStuff.nodes('/PLAY') P(c)

	CROSS APPLY P.c.nodes('PERSONAE/PERSONA') T(c)

-- Estimated cost 40012.3

SELECT
	Francja	= CASE
				WHEN Person.Name LIKE '%FRANCE%'
				THEN Person.Name
				ELSE ''
			END,
	Tytul	= P.c.value('TITLE[1]/text()[1]', 'VARCHAR(100)'),
	Postac	= Person.Name
FROM dbo.PowerShellStuff s

	CROSS APPLY s.XMLStuff.nodes('/PLAY') P(c)

	CROSS APPLY P.c.nodes('PERSONAE/PERSONA') T(c)

	CROSS APPLY (
		SELECT
			T.c.value('text()[1]', 'VARCHAR(100)')
	) Person(Name)

-- 26687.2


SET STATISTICS IO, TIME OFF